const request = require('request-promise');
const searchProduct = require('../api/scrapers/searchproduct');
const fetchProductDetails = require('../api/scrapers/fetchproductdetails');
const login = require('../api/login/login');
const placeBid = require('../api/placebid/index');
const placeAsk = require('../api/placeask/index');
const updateAsk = require('../api/updateask/index');
const updateBid = require('../api/updatebid/index');
const formatProxy = require('../utils/formatproxy');

module.exports = class stockx {
    constructor(options){
        //Configure options
        this.currency = 'USD';
        this.cookieJar = request.jar();
        this.loggedIn = false;

        if (options == undefined) return;

        this.currency = options.currency == undefined ? 'USD' : options.currency;
        this.proxy = options.proxy == undefined || options.proxy.trim() == '' ? '' : formatProxy(options.proxy);
    };

    async searchProducts(query, options){
        try {
            //Search products and return them
            const products = await searchProduct(query, 'product', options, this.proxy);
            return products;
        }
        catch(e){
            throw new Error(e.message);
        };
    };

    async fetchProductDetails(product){
        try {
            //Fetch products and return them
            const products = await fetchProductDetails(product, this.currency, this.proxy);
            return products;
        }
        catch(e){
            throw new Error(e.message);
        };
    };

    async login(user, pass){
        try {
            //Create login
            await login(user, pass, this.cookieJar, this.proxy);

            //Verify a token was created
            this.token = this.cookieJar._jar.store.idx["stockx.com"]["/"].token;
            if (this.token == undefined) throw new Error("No login token found!");

            //Store the account token as a local class variable
            this.token = this.token.toString().split('token=')[1].split(';')[0];
            this.loggedIn = true;
        }
        catch(e){
            throw new Error(e);
        };
    };

    async placeBid(product, amount, requestedSize){
        //Convert amount to digit
        amount = Number(amount);

        //Verify fields passed in by user
        if (!this.loggedIn) throw new Error("You must be logged in before placing a bid!");
        else if (amount == undefined || amount == NaN) throw new Error("Amount is incorrect, please ensure your paramaters are correctly formatted.");
        else if (requestedSize == undefined) throw new Error("Please specify a size to bid on!");
        else if (product == undefined) throw new Error("A product must be specified!");
        else if (typeof product == 'string') throw new Error("The product passed in must an object. Use fetchProductDetails() to get the details first.");
        else if (product.variants == undefined) throw new Error("No variants found in product! Please check the product object passed in.");

        //Get size from requestedSize in the product variants
        const size = requestedSize.toLowerCase() == 'random' ? product.variants[Math.floor(Math.random() * product.variants.length)] : product.variants.find(variant => variant.size == requestedSize);
        
        //Check if getting size was successful
        if (size == undefined) throw new Error("No variant found for the requested size!"); 
        if (size.uuid == undefined || size.uuid == "") throw new Error("No variant ID found for the requested size!");  
        
        //Place bid
        try {
            const response = await placeBid(this.token, amount, size.uuid, this.currency, this.cookieJar, this.proxy);
            return response;
        }
        catch(e){
            throw new Error(e.message);
        };
    };

    async placeAsk(product, amount, requestedSize){
        //Convert amount to digit
        amount = Number(amount);

        //Verify fields passed in by user
        if (!this.loggedIn) throw new Error("You must be logged in before placing a bid!");
        else if (amount == undefined || amount == NaN) throw new Error("Amount is incorrect, please ensure your paramaters are correctly formatted.");
        else if (requestedSize == undefined) throw new Error("Please specify a size to bid on!");
        else if (product == undefined) throw new Error("A product must be specified!");
        else if (typeof product == 'string') throw new Error("The product passed in must an object. Use fetchProductDetails() to get the details first.");
        else if (product.variants == undefined) throw new Error("No variants found in product! Please check the product object passed in.");

        //Get size from requestedSize in the product variants
        const size = requestedSize.toLowerCase() == 'random' ? product.variants[Math.floor(Math.random() * product.variants.length)] : product.variants.find(variant => variant.size == requestedSize);
        
        //Check if getting size was successful
        if (size == undefined) throw new Error("No variant found for the requested size!"); 
        if (size.uuid == undefined || size.uuid == "") throw new Error("No variant ID found for the requested size!");  
        
        //Place bid
        try {
            const response = await placeAsk(this.token, amount, size.uuid, this.currency, this.cookieJar, this.proxy);
            return response;
        }
        catch(e){
            throw new Error(e.message);
        };
    };

    async updateAsk(ask, amount){
        //Convert amount to digit
        amount = Number(amount);

        //Verify fields passed in by user
        if (!this.loggedIn) throw new Error("You must be logged in before placing a bid!");
        else if (amount == undefined || amount == NaN) throw new Error("Amount is incorrect, please ensure your paramaters are correctly formatted.");
        else if (ask == undefined) throw new Error("Ask is incorrect, please ensure your paramaters are correctly formatted.");

        //Get size from previous ask size
        const size = ask.response.PortfolioItem.skuUuid;

        //Check if getting size was successful
        if (size == undefined) throw new Error("No variant found in ask!"); 

        //Update ask
        try {
            const response = await updateAsk(this.token, amount, size, ask.id, this.currency, this.cookieJar, this.proxy);
            return response;
        }
        catch(e){
            throw new Error(e.message);
        };
    };

    async updateBid(bid, amount){
        //Convert amount to digit
        amount = Number(amount);

        //Verify fields passed in by user
        if (!this.loggedIn) throw new Error("You must be logged in before placing a bid!");
        else if (amount == undefined || amount == NaN) throw new Error("Amount is incorrect, please ensure your paramaters are correctly formatted.");
        else if (bid == undefined) throw new Error("Ask is incorrect, please ensure your paramaters are correctly formatted.");

        //Get size from previous ask size
        const size = bid.response.PortfolioItem.skuUuid;

        //Check if getting size was successful
        if (size == undefined) throw new Error("No variant found in ask!"); 

        //Update ask
        try {
            const response = await updateBid(this.token, amount, size, bid.id, this.currency, this.cookieJar, this.proxy);
            return response;
        }
        catch(e){
            throw new Error(e.message);
        };
    };
};